#!/bin/bash

function usage
{
	echo 'ACME Clallenge Uploader v1.0.0 [2020/02/20]'
	echo '(C) 2020 Evgeny Vrublevsky <me@veg.by>'
	echo 'Usage:'
	echo '    acmechal.sh <help|auth|cleanup> [options]'
	echo 'Options:'
	echo '    --mode <auth|cleanup>       Set CERTBOT_MODE.'
	echo '    --password <password>       Set CERTBOT_PASSWORD.'
	echo '    --proto <auto|http|https>   Set CERTBOT_PROTO.'
	echo '    --domain <domain.name>      Set CERTBOT_DOMAIN.'
	echo '    --token <filename>          Set CERTBOT_TOKEN.'
	echo '    --validation <content>      Set CERTBOT_VALIDATION.'
}

if [[ -z "$1" ]] || [[ "$1" = "-h" ]] || [[ "$1" = "--help" ]] || [[ "$1" = "help" ]]; then
	usage
	exit 0
fi

# Parse arguments.

POSITIONAL=()
while [[ $# -gt 0 ]]; do
case "$1" in
	--mode)
		CERTBOT_MODE="$2"
		shift
		shift
		;;
	--password)
		CERTBOT_PASSWORD="$2"
		shift
		shift
		;;
	--protocol)
		CERTBOT_PROTOCOL="$2"
		shift
		shift
		;;
	--domain)
		CERTBOT_DOMAIN="$2"
		shift
		shift
		;;
	--token)
		CERTBOT_TOKEN="$2"
		shift
		shift
		;;
	--validation)
		CERTBOT_VALIDATION="$2"
		shift
		shift
		;;
	*)
		# Unknown option.
		POSITIONAL+=("$1")
		shift
		;;
esac
done

# Parse positional parameters.

set -- "${POSITIONAL[@]}"

if [[ -z "${CERTBOT_MODE}" ]]; then
	CERTBOT_MODE="$1"
fi

# Check if required variables are provided.

if [[ "${CERTBOT_MODE}" != "auth" ]] && [[ "${CERTBOT_MODE}" != "cleanup" ]]; then
	echo 'Error: CERTBOT_MODE should be "auth" or "cleanup".' 1>&2
	exit 1
fi
if [[ -z "${CERTBOT_DOMAIN}" ]] || [[ -z "${CERTBOT_TOKEN}" ]] || [[ -z "${CERTBOT_PASSWORD}" ]]; then
	echo 'Error: CERTBOT_DOMAIN, CERTBOT_TOKEN, and CERTBOT_PASSWORD are required.' 1>&2
	exit 1
fi

# Detect if we can use HTTPS.

if [[ -z "${CERTBOT_PROTO}" ]] || [[ "${CERTBOT_PROTO}" = "auto" ]]; then
	CERTBOT_PROTO=$(echo ^d | openssl s_client -no_ssl3 -connect "${CERTBOT_DOMAIN}:443" -verify 5 -verify_hostname "${CERTBOT_DOMAIN}" -verify_return_error 2>/dev/null 1>/dev/null && echo "https" || echo "http")
fi

# Execute desired action.

if [[ "${CERTBOT_MODE}" = "auth" ]]; then
	curl --silent --show-error --fail --user "acme:${CERTBOT_PASSWORD}" --url "${CERTBOT_PROTO}://${CERTBOT_DOMAIN}/.well-known/acme-challenge/${CERTBOT_TOKEN}" --request PUT --header "Content-Type: text/plain" --data-raw "${CERTBOT_VALIDATION}"
else
	curl --silent --show-error --fail --user "acme:${CERTBOT_PASSWORD}" --url "${CERTBOT_PROTO}://${CERTBOT_DOMAIN}/.well-known/acme-challenge/${CERTBOT_TOKEN}" --request DELETE
fi
