<?php

// ACME Clallenge Handler v1.0.0 [2020/02/20]
// (C) 2020 Evgeny Vrublevsky <me@veg.by>

define('PASSWORD', 'Your_PASSW0RD');
define('DATAFILE', preg_replace('#\.php$#', '', __FILE__) . '.json');
define('KEEPTIME', 3600); // Use 0 to keep forever.
define('DOUNLINK', true); // Delete DATAFILE when data is empty.
define('ACMEPATH', '/.well-known/acme-challenge/');
header('Content-Type: text/plain; charset=utf-8');

class http_exception extends exception {}

function get(&$var, $default = null)
{
	return isset($var) ? $var : $default;
}

function check_password()
{
	if (get($_SERVER['PHP_AUTH_USER']) !== PASSWORD && get($_SERVER['PHP_AUTH_PW']) !== PASSWORD && get($_COOKIE['password']) !== PASSWORD)
	{
		throw new http_exception('Unauthorized', 401);
	}
}

function save_datafile($data)
{
	if (DOUNLINK && empty($data))
	{
		if (file_exists(DATAFILE) && !unlink(DATAFILE))
		{
			throw new http_exception('Internal Server Error', 500);
		}
	}
	else
	{
		$json = json_encode($data, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES);
		if (file_put_contents(DATAFILE, $json, LOCK_EX) === false)
		{
			throw new http_exception('Internal Server Error', 500);
		}
	}
}

function load_datafile()
{
	if (!file_exists(DATAFILE)) { return []; }
	if (KEEPTIME && (time() - filemtime(DATAFILE) > KEEPTIME))
	{
		if (DOUNLINK) { @unlink(DATAFILE); }
		return [];
	}
	$data = json_decode(file_get_contents(DATAFILE), true);
	if (!is_array($data)) { return []; }
	return $data;
}

try
{
	if (strpos($_SERVER['REQUEST_URI'], ACMEPATH) !== 0) { throw new http_exception('Bad Request', 400); }
	$filename = substr(explode('?', $_SERVER['REQUEST_URI'])[0], strlen(ACMEPATH));
	if (!$filename) { throw new http_exception('Forbidden', 403); }
	$data = load_datafile();

	switch ($_SERVER['REQUEST_METHOD'])
	{
	case 'PUT':

		check_password();
		$data[$filename] = file_get_contents('php://input');
		save_datafile($data);
		echo("OK\n");
		break;

	case 'DELETE':

		check_password();
		if (!isset($data[$filename])) { throw new http_exception('Not Found', 404); }
		unset($data[$filename]);
		save_datafile($data);
		echo("OK\n");
		break;

	default:

		if (!isset($data[$filename])) { throw new http_exception('Not Found', 404); }
		echo $data[$filename];
		break;
	}
}
catch (http_exception $e)
{
	$message = $e->getMessage();
	$code = $e->getCode() ? $e->getCode() : 500;
	http_response_code($code);
	echo($code . ' ' . $message . "\n");
}
